/*
 * (C) Copyright 2001
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <ppcboot.h>
#include <config.h>
#include "mpc8xx.h"

/*
 *  Memory Controller Using
 *
 *  CS0 - Flash memory            (0x40000000)
 *  CS1 - SDRAM                   (0x00000000}
 *  CS2 - S/UNI Ultra ATM155
 *  CS3 - IDT 77106 ATM25
 *  CS4 - DSP HPI
 *  CS5 - E1/T1 Interface device
 *  CS6 - PCMCIA device
 *  CS7 - PCMCIA device
 */

/* ------------------------------------------------------------------------- */

#define	_NOT_USED_	0xFFFFFFFF

#if defined(CONFIG_ICU862)
/* 40x2MHz tables */
const uint sdram[]=
{ 0x0e06fc04, 0x11acfc04, 0xefa98c00, 0x1ff77c47,
  0xeeeabc35, 0x11b57fff, 0xffffffff, 0xffffffff,
  0x0e06fc04, 0x10adfc00, 0xf0affc00, 0xf0affc00,
  0xe1bbfc00, 0x1ff77c47, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0x0e26bc04, 0x01b93c00, 0x1ff77c47, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0x0e06bc04, 0x10ad7c00, 0xf0affc00, 0xf0affc00,
  0xe1bbbc04, 0x1ff77c47, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xfffffc04,
  0x0ee27c04, 0x10a0fc84, 0xf0a0fc04, 0xf0a0fc84,
  0xf1b5fc07, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xfffffc07, 0xffffffff, 0xffffffff, 0xffffffff };
#endif

#define MAMR_VALUE	0x27904111
#define MCR_INIT	0x8000222f
#define MAR_VALUE	0x00000088
#define MCR_L_M_R	0x80002105
#define MPTPR_VALUE	0x0200

#define OR1_VALUE	0xfe000900
#define BR1_VALUE	0x00000081

/* ------------------------------------------------------------------------- */


/*
 * Check Board Identity:
 */

int checkboard (void)
{
  printf("ICU862 Board\n");
  return 0;
}

/* ------------------------------------------------------------------------- */
int _draminit(uint base, uint noMbytes, uint edo, uint delay)
{
	return 0;
}

/* ------------------------------------------------------------------------- */

void _dramdisable(void)
{
	volatile immap_t     *immap = (immap_t *)CFG_IMMR;
	volatile memctl8xx_t *memctl = &immap->im_memctl;

	memctl->memc_br2 = 0x00000000;
	memctl->memc_br3 = 0x00000000;

	/* maybe we should turn off upma here or something */
}

int _initsdram(uint base, uint noMbytes)
{
	volatile immap_t     *immap = (immap_t *)CFG_IMMR;
	volatile memctl8xx_t *memctl = &immap->im_memctl;

	if(noMbytes != 32)
	{
		return -1;
	}

	upmconfig(UPMA, (uint *)sdram,sizeof(sdram)/sizeof(uint));


	/* Configure the refresh (mostly).  This needs to be
	* based upon processor clock speed and optimized to provide
	* the highest level of performance.  For multiple banks,
	* this time has to be divided by the number of banks.
	* Although it is not clear anywhere, it appears the
	* refresh steps through the chip selects for this UPM
	* on each refresh cycle.
	* We have to be careful changing
	* UPM registers after we ask it to run these commands.
	*/

	memctl->memc_mamr = MAMR_VALUE;
	memctl->memc_mcr = MCR_INIT;
	memctl->memc_mar = MAR_VALUE;
	memctl->memc_mcr = MCR_L_M_R;

	memctl->memc_mptpr = MPTPR_VALUE;

	udelay(200);

	memctl->memc_or1 = OR1_VALUE;
	memctl->memc_br1 = BR1_VALUE | base;

	return 0;
}

/* ------------------------------------------------------------------------- */

void _sdramdisable(void)
{
	volatile immap_t     *immap = (immap_t *)CFG_IMMR;
	volatile memctl8xx_t *memctl = &immap->im_memctl;

	memctl->memc_br1 = 0x00000000;

	/* maybe we should turn off upmb here or something */
}

/* ------------------------------------------------------------------------- */

int initsdram(uint base, uint *noMbytes)
{
	uint m = 32;

	*noMbytes = m;

	if(!_initsdram(base, m))
	{

		return 0;
	}
	else
	{
		_sdramdisable();

		return -1;
	}
	return 0;
}

long int initdram (int board_type)
{
	uint m;

	if (!initsdram(0x00000000, &m)) {
		printf("(%u MB SDRAM) ", m);
	} else {

		/********************************
		 *DRAM ERROR, HALT PROCESSOR
		 *********************************/
		while(1);

		return -1;
	}

	return (m << 20);
}

/* ------------------------------------------------------------------------- */

int testdram (void)
{
    /* TODO: XXX XXX XXX */
    printf ("test: 16 MB - ok\n");

    return (0);
}

